package com.example.survey.dao.impl;

import com.example.survey.dao.RecordDao;
import com.example.survey.entity.Project;
import com.example.survey.entity.Record;
import com.example.survey.entity.Respondent;
import com.example.survey.entity.User;
import com.example.survey.enumeration.RecordStateEnum;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * @author Pope
 */
@Log4j2
@Repository
public class RecordDaoImpl implements RecordDao {

    @Autowired
    MongoTemplate mongoTemplate;


    @Override
    public boolean existRecord(Respondent respondent, Project project, String state) {
        Criteria criteria = new Criteria()
                .and("respondent.$id").is(respondent.getId())
                .and("project.$id").is(project.getId())
                .and("state").is(state);
        Query query = new Query(criteria);
        return mongoTemplate.exists(query, Record.class);
    }

    @Override
    public Record getRecord(Respondent respondent, Project project, String state) {
        Criteria criteria = new Criteria()
                .and("respondent.$id").is(respondent.getId())
                .and("project.$id").is(project.getId())
                .and("state").is(state);
        Query query = new Query(criteria);

        return mongoTemplate.findOne(query, Record.class);
    }

    @Override
    public void saveRecord(Record record) {
        mongoTemplate.save(record);
    }

    @Override
    public long countRecord(Respondent respondent, User user, Project project, String state, String version, String questionnaireNumber) {
        Criteria criteria = new Criteria();
        if (respondent != null) {
            criteria.and("respondent.$id").is(respondent.getId());
        }
        if (user != null) {
            criteria.and("user.$id").is(user.getId());
        }
        if (project != null) {
            criteria.and("project.$id").is(project.getId());
        }

        if (state != null) {
            criteria.and("state").is(state);
        } else {
            criteria.and("state").in(RecordStateEnum.REVIEWED.getValue(), RecordStateEnum.UNDER_REVIEW.getValue());
        }

        if (version != null) {
            criteria.and("version").is(version);
        }
        if (questionnaireNumber != null) {
            criteria.and("value.questionnaireNumber").is(questionnaireNumber);
        }
        Query query = new Query(criteria);
        return mongoTemplate.count(query, Record.class);
    }

    @Override
    public List<Record> listRecordLimit(Respondent respondent, User user, Project project, String state, String version, String questionnaireNumber, int offset, int pageSize) {
        Criteria criteria = new Criteria();
        if (respondent != null) {
            criteria.and("respondent.$id").is(respondent.getId());
        }
        if (user != null) {
            criteria.and("user.$id").is(user.getId());
        }
        if (project != null) {
            criteria.and("project.$id").is(project.getId());
        }

        if (state != null) {
            criteria.and("state").is(state);
        } else {
            criteria.and("state").in(RecordStateEnum.REVIEWED.getValue(), RecordStateEnum.UNDER_REVIEW.getValue());
        }

        if (version != null) {
            criteria.and("version").is(version);
        }
        if (questionnaireNumber != null) {
            criteria.and("value.questionnaireNumber").is(questionnaireNumber);
        }
        Query query = new Query(criteria).skip(offset).limit(pageSize);
        return mongoTemplate.find(query, Record.class);
    }

    @Override
    public Record selectRecord(Respondent respondent, Project project, String version) {
        Criteria criteria = Criteria
                .where("respondent.$id").is(respondent.getId())
                .and("project.$id").is(project.getId());
        if (version != null) {
            criteria.and("version").is(version);
        }
        Query query = new Query(criteria);
        return mongoTemplate.findOne(query, Record.class);
    }

    @Override
    public Record selectRecord(Respondent respondent, Project project, List<String> states) {
        Criteria criteria = new Criteria()
                .and("respondent.$id").is(respondent.getId())
                .and("project.$id").is(project.getId())
                .and("state").in(states);
        Query query = new Query(criteria);
        return mongoTemplate.findOne(query, Record.class);
    }

    @Override
    public List<Record> listRecord(Project project, Respondent respondent) {

        Query query = new Query(Criteria.where("project.$id").is(project.getId())
                .and("respondent.$id").is(respondent.getId()));
        return mongoTemplate.find(query,Record.class);
    }
}
