package com.example.survey.service.impl;

import com.example.survey.dao.*;
import com.example.survey.dto.record.*;
import com.example.survey.entity.*;
import com.example.survey.entity.inner.Operation;
import com.example.survey.enumeration.RecordStateEnum;
import com.example.survey.enumeration.RespondentStateEnum;
import com.example.survey.enumeration.ResultEnum;
import com.example.survey.exception.*;
import com.example.survey.service.RecordService;
import com.example.survey.util.WordUtil;
import com.example.survey.vo.RecordVO;
import com.example.survey.vo.inner.OperationInfo;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Collectors;

/**
 * @author Pope
 */
@Log4j2
@Service
public class RecordServiceImpl implements RecordService {


    @Autowired
    private RecordDao recordDao;

    @Autowired
    private RespondentDao respondentDao;

    @Autowired
    private UserDao userDao;

    @Autowired
    private ProjectDao projectDao;

    @Autowired
    private MetaDataDao metaDataDao;

    @Value("${file.path}")
    private String path;

    @Value("${file.url}")
    private String url;

    @Override
    public long countRecord(String userPhone, String projectName) {
        if (userPhone != null && !userDao.existUser(userPhone)) {
            throw new UserException(ResultEnum.NOT_EXIST_USER);
        }
        if (projectName != null && !projectDao.existProject(projectName)) {
            throw new ProjectException(ResultEnum.NOT_EXIST_PROJECT);
        }
        User user = userDao.selectUser(userPhone);
        Project project = projectDao.selectProject(projectName);
        return recordDao.countRecord(null, user, project, RecordStateEnum.UNDER_REVIEW.getValue(), null, null);
    }

    @Override
    public void reviewRecord(ReviewRecordDTO reviewRecordDTO) {
        if (!userDao.existUser(reviewRecordDTO.getReviewerPhone())) {
            throw new UserException(ResultEnum.NOT_EXIST_USER);
        }
        User user = userDao.selectUser(reviewRecordDTO.getReviewerPhone());
        if (!projectDao.existProject(reviewRecordDTO.getProjectName())) {
            throw new ProjectException(ResultEnum.NOT_EXIST_PROJECT);
        }
        Project project = projectDao.selectProject(reviewRecordDTO.getProjectName());
        if (!respondentDao.existRespondent(reviewRecordDTO.getIdNumber(), project, RespondentStateEnum.INVESTIGATED.getValue())) {
            throw new RespondentException(ResultEnum.NOT_EXIST_RESPONDENT);
        }
        Respondent respondent = respondentDao.selectRespondent(reviewRecordDTO.getIdNumber(), project);
        if (!recordDao.existRecord(respondent, project, RecordStateEnum.UNDER_REVIEW.getValue())) {
            throw new RecordException(ResultEnum.NOT_EXIST_RECORD);
        }
        Record record = recordDao.getRecord(respondent, project, RecordStateEnum.UNDER_REVIEW.getValue());


        record.setState(reviewRecordDTO.getPass() ? RecordStateEnum.REVIEWED.getValue() : RecordStateEnum.NOT_PASS.getValue());
        record.setVersion(UUID.randomUUID().toString());
        Operation reviewOp = Operation.reviewOp(user, reviewRecordDTO.getMsg(), record.getVersion(), reviewRecordDTO.getPass());
        List<Operation> opList = record.getOperationList();
        opList.add(reviewOp);
        record.setOperationList(opList);

        recordDao.saveRecord(record);
    }

    @Override
    public void modifyRecord(ModifyRecordDTO modifyRecordDTO) {
        if (!userDao.existUser(modifyRecordDTO.getUserPhone())) {
            throw new UserException(ResultEnum.NOT_EXIST_USER);
        }
        User user = userDao.selectUser(modifyRecordDTO.getUserPhone());
        if (!projectDao.existProject(modifyRecordDTO.getProjectName())) {
            throw new ProjectException(ResultEnum.NOT_EXIST_PROJECT);
        }
        Project project = projectDao.selectProject(modifyRecordDTO.getProjectName());
        if (!respondentDao.existRespondent(modifyRecordDTO.getIdNumber(), project, RespondentStateEnum.INVESTIGATED.getValue())) {
            throw new RespondentException(ResultEnum.NOT_EXIST_RESPONDENT);
        }
        Respondent respondent = respondentDao.selectRespondent(modifyRecordDTO.getIdNumber(), project);

        List<String> stateList = new ArrayList<>();
        stateList.add(RecordStateEnum.UNDER_REVIEW.getValue());
        stateList.add(RecordStateEnum.REVIEWED.getValue());
        Record record = recordDao.selectRecord(respondent, project, stateList);
        if (record == null) {
            throw new RecordException(ResultEnum.NOT_EXIST_RECORD);
        }

        String newVersion = UUID.randomUUID().toString();
        List<Operation> oldOperationList = record.getOperationList();
        List<Operation> newOperationList = record.getOperationList();

        Operation coverOp = Operation.coverOp(user, newVersion);
        oldOperationList.add(coverOp);

        Operation modifyOp = Operation.modifyOp(user, modifyRecordDTO.getMsg(), newVersion);
        newOperationList.add(modifyOp);

        record.setOperationList(oldOperationList);
        record.setState(RecordStateEnum.FILED.getValue());
        recordDao.saveRecord(record);

        record.setId(null);
        record.setValues(modifyRecordDTO.getValues());
        record.setOperationList(newOperationList);
        record.setState(RecordStateEnum.UNDER_REVIEW.getValue());

        recordDao.saveRecord(record);

    }

    @Override
    public void createRecord(SubmitRecordDTO submitRecordDTO) {
        if (!userDao.existUser(submitRecordDTO.getUserPhone())) {
            throw new UserException(ResultEnum.NOT_EXIST_USER);
        }
        User user = userDao.selectUser(submitRecordDTO.getUserPhone());
        if (!projectDao.existProject(submitRecordDTO.getProjectName())) {
            throw new ProjectException(ResultEnum.NOT_EXIST_PROJECT);
        }
        Project project = projectDao.selectProject(submitRecordDTO.getProjectName());
        if (!metaDataDao.existMetaData(submitRecordDTO.getMetaDataName())) {
            throw new MetaDataException(ResultEnum.NOT_EXIST_METADATA);
        }
        MetaData metaData = metaDataDao.selectMetaData(submitRecordDTO.getMetaDataName());
        if (!respondentDao.existRespondent(submitRecordDTO.getIdNumber(), project)) {
            throw new RespondentException(ResultEnum.NOT_EXIST_RESPONDENT);
        }
        Respondent respondent = respondentDao.selectRespondent(submitRecordDTO.getIdNumber(), project);
        //设置调查对象为已调查
        respondent.setState(RespondentStateEnum.INVESTIGATED.getValue());
        respondentDao.saveRespondent(respondent);

        List<String> stateList = new ArrayList<>();
        stateList.add(RecordStateEnum.UNDER_REVIEW.getValue());
        stateList.add(RecordStateEnum.REVIEWED.getValue());
        Record record = recordDao.selectRecord(respondent, project, stateList);

        String newVersion = UUID.randomUUID().toString();
        List<Operation> oldOperationList = new ArrayList<>();
        List<Operation> newOperationList = new ArrayList<>();

        if (record != null) {
            oldOperationList = record.getOperationList();
            newOperationList = record.getOperationList();
            Operation coverOp = Operation.coverOp(user, newVersion);
            oldOperationList.add(coverOp);
            record.setOperationList(oldOperationList);
            record.setState(RecordStateEnum.FILED.getValue());
            recordDao.saveRecord(record);
        } else {
            record = new Record();
        }

        record.setProject(project);
        record.setUser(user);
        record.setValues(submitRecordDTO.getValues());
        Operation submitOp = Operation.submitOp(user, submitRecordDTO.getMsg(), newVersion);
        newOperationList.add(submitOp);
        record.setOperationList(newOperationList);
        record.setState(RecordStateEnum.UNDER_REVIEW.getValue());
        record.setMetaData(metaData);
        record.setVersion(newVersion);
        recordDao.saveRecord(record);
    }

    @Override
    public long countRecord(String idNumber, String userPhone, String projectName, String state, String version, String questionnaireNumber) {
        if (projectName != null && !projectDao.existProject(projectName)) {
            throw new ProjectException(ResultEnum.NOT_EXIST_PROJECT);
        }
        Project project = projectDao.selectProject(projectName);

        if (userPhone != null && !userDao.existUser(userPhone)) {
            throw new UserException(ResultEnum.NOT_EXIST_USER);
        }
        User user = userDao.selectUser(userPhone);

        if (idNumber != null && !respondentDao.existRespondent(idNumber, project)) {
            throw new RespondentException(ResultEnum.NOT_EXIST_RESPONDENT);
        }

        Respondent respondent = respondentDao.selectRespondent(idNumber, project);


        return recordDao.countRecord(respondent, user, project, state, version, questionnaireNumber);
    }

    @Override
    public List<RecordVO> listRecordLimit(String idNumber, String userPhone, String projectName, String state, String version, String questionnaireNumber, int currentPage, int pageSize) {
        if (userPhone != null && !userDao.existUser(userPhone)) {
            throw new UserException(ResultEnum.NOT_EXIST_USER);
        }
        User user = userDao.selectUser(userPhone);

        if (projectName != null && !projectDao.existProject(projectName)) {
            throw new ProjectException(ResultEnum.NOT_EXIST_PROJECT);
        }
        Project project = projectDao.selectProject(projectName);

        if (idNumber != null && !respondentDao.existRespondent(idNumber, project)) {
            throw new RespondentException(ResultEnum.NOT_EXIST_RESPONDENT);
        }
        Respondent respondent = respondentDao.selectRespondent(idNumber, project);


        List<Record> recordList = recordDao.listRecordLimit(respondent, user, project, state, version, questionnaireNumber, currentPage * pageSize, pageSize);
        return recordList.stream().map(record -> {
            RecordVO recordVO = new RecordVO();
            recordVO.setIdNumber(record.getRespondent().getIdNumber());
            recordVO.setUserPhone(record.getUser().getPhone());
            recordVO.setProjectName(record.getProject().getName());
            recordVO.setOperationInfoList(record.getOperationList().stream().map(op -> {
                OperationInfo operationInfo = new OperationInfo();
                operationInfo.setType(op.getType());
                operationInfo.setTime(op.getTime());
                operationInfo.setUserPhone(op.getUser().getPhone());
                operationInfo.setMsg(op.getMsg());
                operationInfo.setResult(op.getResult());
                operationInfo.setVersion(op.getVersion());
                return operationInfo;
            }).collect(Collectors.toList()));
            recordVO.setVersion(record.getVersion());
            recordVO.setState(record.getState());
            return recordVO;
        }).collect(Collectors.toList());
    }

    @Override
    public Map<String, Object> getRecordValues(String projectName, String idNumber, String version) {
        if (!projectDao.existProject(projectName)) {
            throw new ProjectException(ResultEnum.NOT_EXIST_PROJECT);
        }
        Project project = projectDao.selectProject(projectName);
        if (!respondentDao.existRespondent(idNumber, project)) {
            throw new RespondentException(ResultEnum.NOT_EXIST_RESPONDENT);
        }
        Respondent respondent = respondentDao.selectRespondent(idNumber, project);
        Record record = recordDao.selectRecord(respondent, project, version);
        if (record == null) {
            throw new RecordException(ResultEnum.NOT_EXIST_RECORD);
        }
        return record.getValues();
    }

    @Override
    public String uploadFile(MultipartFile file) {
        String name = file.getOriginalFilename();
        assert name != null;
        String suffix = name.substring(name.lastIndexOf('.'));
        String newName = UUID.randomUUID().toString() + suffix;
        File newFile = new File(path + newName);

        OutputStream os = null;
        InputStream is = null;

        try {
            os = new FileOutputStream(newFile);
            is = file.getInputStream();

            byte[] buffer = new byte[1024];
            int len;
            while ((len = is.read(buffer)) != -1) {
                os.write(buffer, 0, len);
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (os != null) {
                    os.close();
                }
                if (is != null) {
                    is.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return url + newName;
    }

    @Override
    public void deleteRecord(DeleteRecordDTO deleteRecordDTO) {
        if (!userDao.existUser(deleteRecordDTO.getUserPhone())) {
            throw new UserException(ResultEnum.NOT_EXIST_USER);
        }
        User user = userDao.selectUser(deleteRecordDTO.getUserPhone());
        if (!projectDao.existProject(deleteRecordDTO.getProjectName())) {
            throw new ProjectException(ResultEnum.NOT_EXIST_PROJECT);
        }
        Project project = projectDao.selectProject(deleteRecordDTO.getProjectName());
        if (!respondentDao.existRespondent(deleteRecordDTO.getIdNumber(), project)) {
            throw new RespondentException(ResultEnum.NOT_EXIST_RESPONDENT);
        }
        Respondent respondent = respondentDao.selectRespondent(deleteRecordDTO.getIdNumber(), project);

        List<String> stateList = new ArrayList<>();
        stateList.add(RecordStateEnum.UNDER_REVIEW.getValue());
        stateList.add(RecordStateEnum.REVIEWED.getValue());
        stateList.add(RecordStateEnum.NOT_PASS.getValue());
        Record record = recordDao.selectRecord(respondent, project, stateList);
        if (record == null) {
            throw new RecordException(ResultEnum.NOT_EXIST_RECORD);
        }
        List<Operation> operationList = record.getOperationList();
        Operation deleteOp = Operation.deleteOp(user, deleteRecordDTO.getMsg(), record.getVersion());
        operationList.add(deleteOp);
        record.setOperationList(operationList);
        record.setState(RecordStateEnum.FILED.getValue());
        recordDao.saveRecord(record);
    }

    @Override
    public void record2word(String idNumber, String projectName, HttpServletResponse response) {
        //TODO 导出为word
    }

    @Override
    public void modifyProject(ModifyProjectDTO modifyProjectDTO) {
        if (!projectDao.existProject(modifyProjectDTO.getProjectName())) {
            throw new ProjectException(ResultEnum.NOT_EXIST_PROJECT);
        }
        Project project = projectDao.selectProject(modifyProjectDTO.getProjectName());
        if(!projectDao.existProject(modifyProjectDTO.getNewProjectName())){
            throw new ProjectException(ResultEnum.NOT_EXIST_PROJECT);
        }
        Project newProject = projectDao.selectProject(modifyProjectDTO.getNewProjectName());
        if (!respondentDao.existRespondent(modifyProjectDTO.getIdNumber(), project)) {
            throw new RespondentException(ResultEnum.NOT_EXIST_RESPONDENT);
        }
        Respondent respondent = respondentDao.selectRespondent(modifyProjectDTO.getIdNumber(), project);

        List<String> states = new ArrayList<>();
        states.add(RecordStateEnum.REVIEWED.getValue());
        states.add(RecordStateEnum.UNDER_REVIEW.getValue());
        Record record = recordDao.selectRecord(respondent, project, states);

        //如果新项目已经存在调查对象则覆盖
        respondent.setId(null);
        respondent.setPhone(modifyProjectDTO.getPhone());
        respondent.setName(modifyProjectDTO.getName());
        respondent.setMsg(modifyProjectDTO.getMsg());
        respondent.setGender(modifyProjectDTO.getGender());
        respondent.setAdministrativeArea(modifyProjectDTO.getAdministrativeArea());
        respondent.setProject(newProject);
        if(respondentDao.existRespondent(modifyProjectDTO.getIdNumber(),newProject)){
            respondent.setId(respondentDao.selectRespondent(modifyProjectDTO.getIdNumber(), project).getId());
        }
        respondentDao.saveRespondent(respondent);

        record.setRespondent(respondent);
        record.setProject(project);
        recordDao.saveRecord(record);
    }

    @Override
    public void modifyMetaData(ModifyMetaDataDTO modifyMetaDataDTO) {
        if (!projectDao.existProject(modifyMetaDataDTO.getProjectName())) {
            throw new ProjectException(ResultEnum.NOT_EXIST_PROJECT);
        }
        Project project = projectDao.selectProject(modifyMetaDataDTO.getProjectName());
        if (!metaDataDao.existMetaData(modifyMetaDataDTO.getMetaDataName())) {
            throw new MetaDataException(ResultEnum.NOT_EXIST_METADATA);
        }
        MetaData metaData = metaDataDao.selectMetaData(modifyMetaDataDTO.getMetaDataName());
        if (!respondentDao.existRespondent(modifyMetaDataDTO.getIdNumber(), project)) {
            throw new RespondentException(ResultEnum.NOT_EXIST_RESPONDENT);
        }
        Respondent respondent = respondentDao.selectRespondent(modifyMetaDataDTO.getIdNumber(), project);

        List<String> states = new ArrayList<>();
        states.add(RecordStateEnum.REVIEWED.getValue());
        states.add(RecordStateEnum.UNDER_REVIEW.getValue());
        Record record = recordDao.selectRecord(respondent, project, states);
        record.setMetaData(metaData);
        recordDao.saveRecord(record);
    }


}
